################################################################################
## 
## Script to probabilistically and subsequently forecast the condition states 
## of a sewer pipe network based on the deterioration model described in 
## Egger et al. (2013) and a sample of deterioration parameters. 
## Multiple realizations can be computed to reflect the random nature of the 
## model.
##
## The probability that a pipe section in condition state (CS) i will change 
## to CS i+1 within the time step from age t to t+delta is calculated with
## Eq. (6) given in Scheidegger, A., Hug, T., Rieckermann, J. and Maurer, M. 
## (2011) Network condition simulator for benchmarking sewer deterioration 
## models. Water Research 45(16), 4983-4994.
##
## Please contact Christoph Egger, christoph.egger@hotmail.com for support
## and comments.

## Empty the workspace:
rm(list = ls())

################################################################################
##
## 1.) Please make the following specifications

## Specify name of the dataset. For the forecast, the "incomplete" dataset is
## loaded. This dataset includes exclusively pipe which are still in service.
## Only these pipes are of interest.

dataset="dataset_example_incomplete"

## Specify name of inference
inference="example"

## Specify the amount of burnin samples. These samples are discarded

amount.burnin.samples=10000

## Specify the calculation time step (0.1 is adequate)
delta=0.1

## Specify the simulation time steps in years, i.e. the time steps for 
## which subsequently forecasted condition states are saved.
delta_t=1

## Specify forecast horizon in amount of years.
forecast.horizon=30

## Specify the number of simulations to be realized.
n.sim=1000

## Specify the directory where model functions are located

path.model="C:/SDM/functions"

## Specify the directory where the pipe condition data is located

path.input.data="C:/SDM/input"

## Specify the directory of the result folder

path.result="C:/SDM/result"


###########################################################################
##
## 2.) Load functions needed for inference

setwd(path.model)

source("SDM_functions model.R")

###########################################################################
##
## 3.) Loads sample of deterioration parameters. 

setwd(path.input.data)

file.denotation=paste(dataset, "_", inference, sep="")

result.file.name.Rdata=paste("result_", file.denotation, ".Rdata", sep="")

load(file=result.file.name.Rdata)

## The parameters of the burnin phase are not used for predictions.
## For the forecast of CS only the deterioration parameters alpha, beta and 
## mju are relevant and therefore selected.

samp.selected=samp$sample[(amount.burnin.samples+1):nrow(samp$sample),1:3]

###########################################################################
##
## 4.) Loads pipe condition data.

setwd(path.input.data)

filename.dataset=paste(dataset, ".RData", sep="")

load(filename.dataset)

###########################################################################
##
## 5.) Makes (multiple) probabilistic forecasts of condition states.

## Specifies time step in years for stepwise prognosis.
delta_t_future=rep(delta_t, nrow(data.set))

## Number of forecast steps
forecast.steps=forecast.horizon/ delta_t

# Creates a vector of timestep names with the length (forecast.horizon+1)
colnames.year=rep("y ", (forecast.steps+1))
for (i in 1:(forecast.steps+1))
  colnames.year[i]=paste(colnames.year[i], (i-1), sep="")

## Save forecasted CS in a list containing predicted CS for every pipe
## and forecast step. Every component of the list represents one 
## realization. 

list.matrix=
  as.data.frame(matrix(NaN, nrow=nrow(data.set), ncol=forecast.steps+2))

## Specifies a pipe ID for every pipe.

list.matrix[,1]=1:nrow(data.set)

colnames(list.matrix)=c("Pipe_ID", colnames.year)

condition.list=list()

setwd(path.result)

## Generates n.sim realizations of forecasted CS.

for (k in 1:n.sim)
{
  ## Create input data set for the forecast
  forecast.input=data.set
  # Specifies additional variables
  forecast.input$delta_t=delta_t_future
  
  ## Set here the condition state and pipe age at the 
  ## last available inspection as start variables of the forecast. 
  ## In this specific example the condition state observed at 
  ## the pipe ages tau2 (or tau1 if only one observed CS is available) 
  ## corresponds to the current CS respective the CS at age D, 
  ## i.e.tau2=D.
  ##
  ## In applications where tau2<D, a forecast of CS needs to 
  ## be made over the time period between tau2 and D. This 
  ## feature is not included here.
    
  for (i in 1:nrow(forecast.input))
  {
    if (!is.na(data.set$CS2[i]))
    {
      forecast.input$CS_t_minus1[i]=data.set$CS2[i]
      forecast.input$t_minus1[i]=data.set$tau2[i]
    } else {
      forecast.input$CS_t_minus1[i]=data.set$CS1[i]
      forecast.input$t_minus1[i]=data.set$tau1[i]
    }
  }
  
  ## Specifies further variables, i.e. pipe age and
  ## CS in the current time step to be calculated. 
  
  forecast.input$t=NA
  forecast.input$CS_t=NA
  
  ## Selects randomly one deterioration parameter vector from a given sample for
  ## one forecast to be realized
  
  theta=samp.selected[sample(x=1:(max=nrow(samp.selected)), size=1, replace=TRUE),]
  
  ## Writes current CS into the condition list
  condition.list[[k]]=list.matrix
  condition.list[[k]][,2]=forecast.input$CS_t_minus1
  
  ## Runs the forecast for the first simulation time step
  forecast=FN.forecast.rehab(forecast.input, theta, FN.assign.CS)
  
  condition.list[[k]][,3]=forecast$CS_t
  
  ## Runs the forecast for the remaining time steps
  for (j in 2:forecast.horizon)
  {
    forecast.input=forecast
    forecast.input$t_minus1=forecast$t_minus1+forecast$delta_t
    forecast.input$delta_t=delta_t_future
    forecast.input$CS_t_minus1=forecast$CS_t
    forecast.input$CS_t=NaN
    
    forecast=FN.forecast.rehab(forecast.input, theta, FN.assign.CS)
    
    ## Writes the result into the list of condition states.
    condition.list[[k]][,(j+2)]=forecast$CS_t
  }
  save(file="forecast.RData", list=c("condition.list"))
}

