################################################################################
##
## Log-likelihood
## ==============
## Function which calculates the log of the likelihood that a pipe is in the 
## observed condition state (CS) at tau1 (or tau2) given the pipe has not been 
## replaced and given the parameters theta. 
##
## Different versions of the likelihood exist:
##
## 1.) Conditioned log-likelihood according to Eq. 6 in Egger et al. (2013)
##     for one CS observed at age tau. 
## 
## 
## -----------------------------------------------------------------------------
##
## Probability that a pipe is in CS i given it has not been replaced 
## before tau=T1=x[1]. (corresponds to the first faktor in the numerator of 
## Equation 7 in Egger et al. 2013)


FN.log.L.CS.nr=function(x, par,...)  
   {
   
   ## Translate from new into old data format:
   y=x
   x[1]=y[1]
   x[3]=y[3]
   x[2]=y[4]
   x[4]=y[5]
   theta=par[1:(length(par)/2)]
   lambda=par[(length(par)/2+1):(length(par))]

   S1=FN.S.1(x[1], theta)
   S2=FN.S.2(x[1], theta, S1)
   
   ## Numerator of Eq. 7 in Egger et al (2013) (joint probabilities that a pipe has 
   ## not been replaced before D=T1=x[1] given CS i=x[2] and the probability that a 
   ## pipe is in CS i).
   
   P.nr.CS1.P.CS1=(FN.P.nr.CS1(lambda, T_stern=x[2], TR=x[3]))*S1
   P.nr.CS2.P.CS2=(FN.P.nr.CS2(theta, lambda, T1=x[1], T_stern=x[2], TR=x[3]))*(S2-S1)
   P.nr.CS3.P.CS3=(FN.P.nr.CS3(theta, lambda, T1=x[1], T_stern=x[2], TR=x[3]))*(1-S2)

   ## calculation of the normalization factor which is the probability that a pipe
   ## has not been replaced before T1 independent of the CS:
   
   log.norm.factor=log(P.nr.CS1.P.CS1+P.nr.CS2.P.CS2+P.nr.CS3.P.CS3)
   
   if (x[4]==1) log.P.CS.nr=log(P.nr.CS1.P.CS1)-log.norm.factor
   if (x[4]==2) log.P.CS.nr=log(P.nr.CS2.P.CS2)-log.norm.factor
   if (x[4]==3) log.P.CS.nr=log(P.nr.CS3.P.CS3)-log.norm.factor

   return(log.P.CS.nr)
}

#########################################################################
##
##
## "Conditioned" log-likelihood for one pipe with one or two observed 
## pipe conditions corresponding to Eq. 6 in Egger et al. (2013) except 
## for the possibility to either use one or two subsequently observed 
## condition states.

## One row x in the dataset representing one pipe used for inference has the
## form tau1, tau2, R, D, CS1, CS2. For pipes with only one CS observed
## tau2 and CS2 are set to NA.

FN.conditioned.log.likelihood.CS2=function(x, par,...)
{
  x=as.matrix(x)
  theta.det=par[1:(length(par)/2)]
  lambda=par[(length(par)/2+1):(length(par))]
  
  ## For the normalisation in Eq. 6, all possible cases 1-13 need to 
  ## be considered.
  
  ## calculation of the normalization factor which is the probability that a 
  ## pipe has not been replaced before T1 independent of the observed CS
  
  ## if only one observed CS is available:
  if (is.na(x[6]))
  {
    ## First, the "unconditioned" likelihood is calculated (2. factor in
    ## the numerator of Eq. 6):
    P.CS.1=FN.S.1(t=x[1], theta=theta.det)
    P.CS.2=FN.S.2(t=x[1], theta=theta.det, S1=P.CS.1)- P.CS.1
    P.CS.3=1-P.CS.2-P.CS.1
    
    if (x[3]<x[1])
    {
      P.CS1.nr.num=FN.P.nr.CS1(lambda, T_stern=x[4], TR=x[3])*P.CS.1
      P.CS2.nr.num=FN.P.nr.CS2(theta=theta.det, lambda, T1=x[1],
                               T_stern=x[4], TR=x[3])*P.CS.2
      P.CS3.nr.num=FN.P.nr.CS3(theta=theta.det, lambda, T1=x[1],
                               T_stern=x[4], TR=x[3])*P.CS.3
      
    }   else
    {
      P.CS1.nr.num=FN.P.nr.CS1(lambda, T_stern=x[4], TR=x[3])*P.CS.1
      P.CS2.nr.num=exp(-lambda[2]*(x[4]-x[3]))*P.CS.2
      P.CS3.nr.num=exp(-lambda[3]*(x[4]-x[3]))*P.CS.3
    }
    log.normalization=log(P.CS1.nr.num+ P.CS2.nr.num+P.CS3.nr.num)
    
    if (x[5]==1) log.P.CS.nr=log(P.CS1.nr.num)-log.normalization
    if (x[5]==2) log.P.CS.nr=log(P.CS2.nr.num)-log.normalization
    if (x[5]==3) log.P.CS.nr=log(P.CS3.nr.num)-log.normalization
  }  
  
  ## if two observed CS are available:
  if (!is.na(x[6]))
  {
    ## First, the "unconditioned" likelihood is calculated (2. factor in
    ## the numerator of Eq. 6):
    P.CS.1.1=FN.S.1(t=x[2], theta=theta.det)
    alpha=theta.det[1]
    beta=theta.det[2]
    mju=theta.det[3]
    tau1=x[1]
    tau2=x[2]
    P.CS.1.2=FN.P.1.2(alpha, beta, mju, tau1, tau2)
    P.CS.1.3=FN.P.1.3(alpha, beta, mju, tau1, tau2)
    P.CS.2.3=FN.P.2.3(alpha, beta, mju, tau1, tau2)
    P.CS.2.2=FN.P.2.2(alpha, beta, mju, tau1, tau2)
    P.CS.3.3=FN.P.3.3(alpha, beta, mju, tau1, tau2)
    
    ## Calculation of the numerator of Eq. 7
    ## if R<tau1, cases 3, 5, 7, 9, 11 and 13 are relevant:
    if (x[3]<x[1])
    {
      P.CS1.CS1.nr.num=FN.P.nr.CS1(lambda, T_stern=x[4], TR=x[3])*P.CS.1.1
      
      P.CS1.CS2.nr.num=FN.P.nr.case.5(theta.det, lambda, tau1=x[1], 
                                      tau2=x[2], D=x[4], R=x[3])*P.CS.1.2
      
      P.CS1.CS3.nr.num=FN.P.nr.case.7(theta.det, lambda, tau1=x[1], 
                                      tau2=x[2], D=x[4], R=x[3])*P.CS.1.3
      
      P.CS2.CS2.nr.num=FN.P.nr.case.9(theta.det, lambda, tau1=x[1], 
                                      tau2=x[2], D=x[4], R=x[3])*P.CS.2.2
      
      P.CS2.CS3.nr.num=FN.P.nr.case.11(theta.det, lambda, tau1=x[1], 
                                       tau2=x[2], D=x[4], R=x[3])*P.CS.2.3
      
      P.CS3.CS3.nr.num=FN.P.nr.case.13(theta.det, lambda, tau1=x[1], 
                                       tau2=x[2], D=x[4], R=x[3])*P.CS.3.3
      
      
    }
    ## if tau1<=R<tau2, cases 2 (same as 3), 4, 6, 8, 10, 12 are relevant:
    if (x[1]<=x[3] & x[3]<x[2])
    {
      P.CS1.CS1.nr.num=FN.P.nr.CS1(lambda, T_stern=x[4], TR=x[3])*P.CS.1.1
      
      P.CS1.CS2.nr.num=FN.P.nr.case.4(theta.det, lambda, tau1=x[1], 
                                      tau2=x[2], D=x[4], R=x[3])*P.CS.1.2
      
      P.CS1.CS3.nr.num=FN.P.nr.case.6(theta.det, lambda, tau1=x[1], 
                                      tau2=x[2], D=x[4], R=x[3])*P.CS.1.3
      
      P.CS2.CS2.nr.num=FN.P.nr.case.8(lambda, D=x[4], R=x[3])*P.CS.2.2
      
      P.CS2.CS3.nr.num=FN.P.nr.case.10(theta.det, lambda, tau1=x[1], 
                                       tau2=x[2], D=x[4], R=x[3])*P.CS.2.3
      
      P.CS3.CS3.nr.num=FN.P.nr.case.12(lambda, D=x[4], R=x[3])*P.CS.3.3     
    }
    
    ## if R>=tau2, case 1 is relevant
    if (x[3]>=x[2])
    {
      P.CS1.CS1.nr.num=FN.P.nr.CS1(lambda, T_stern=x[4], TR=x[3])*P.CS.1.1
      
      P.CS1.CS2.nr.num=exp(-lambda[2]*(x[4]-x[3]))*P.CS.1.2
      
      P.CS1.CS3.nr.num=exp(-lambda[3]*(x[4]-x[3]))*P.CS.1.3
      
      P.CS2.CS2.nr.num=exp(-lambda[2]*(x[4]-x[3]))*P.CS.2.2
      
      P.CS2.CS3.nr.num=exp(-lambda[3]*(x[4]-x[3]))*P.CS.2.3
      
      P.CS3.CS3.nr.num=exp(-lambda[3]*(x[4]-x[3]))*P.CS.3.3
    }  
    
    log.normalization=log(P.CS1.CS1.nr.num+ P.CS1.CS2.nr.num+P.CS1.CS3.nr.num+
                            P.CS2.CS2.nr.num+P.CS2.CS3.nr.num+P.CS3.CS3.nr.num) 
    
    if (x[5]==1 & x[6]==1) log.P.CS.nr=log(P.CS1.CS1.nr.num)-log.normalization
    if (x[5]==1 & x[6]==2) log.P.CS.nr=log(P.CS1.CS2.nr.num)-log.normalization
    if (x[5]==1 & x[6]==3) log.P.CS.nr=log(P.CS1.CS3.nr.num)-log.normalization
    if (x[5]==2 & x[6]==2) log.P.CS.nr=log(P.CS2.CS2.nr.num)-log.normalization
    if (x[5]==2 & x[6]==3) log.P.CS.nr=log(P.CS2.CS3.nr.num)-log.normalization
    if (x[5]==3 & x[6]==3) log.P.CS.nr=log(P.CS3.CS3.nr.num)-log.normalization  
    
  }
  return(log.P.CS.nr)   
}


## Calculates the log-likelihood of a data set with N pipes and
## tau=T1=x[1], CS=x[2] and T_stern=x[3]

FN.log.likelihood=function(par, data.set)
  {
  if (any(par[!is.na(par)]<0)) return(-Inf)
  ll=sum(apply(data.set,1,FN.log.L.CS.nr,par=par))
  #ll=ll+sum(apply(data.set,1,FN.log.L.CS.nr,par=par, lambda=lambda))
  return(ll)
  }

## Same log liklihood as above but for using the likelihood directly for 
## inference instead of the log.posterior:

FN.log.likelihood.for.inference=function(par, prior.mean, data.set)
{
  variables=c(par, prior.mean[4:6])
  if (any(variables[!is.na(variables)]<0)) return(-Inf)
  ll=sum(apply(data.set,1,FN.log.L.CS.nr,par=variables))
  #ll=ll+sum(apply(data.set,1,FN.log.L.CS.nr,par=par, lambda=lambda))
  return(ll)
}

## Calculates the log-likelihood. Applied to a dataset with N pipes with
## one or two observed condition states. The dataset has the form
## tau1=x[1], tau2=x[2], R=x[3], D=x[4], CS1=x[5], CS2=[6]

FN.log.likelihood.2CS=function(par, data.set)
{
  if (any(par[!is.na(par)]<0)) return(-Inf)
  ll=sum(apply(data.set,1,FN.conditioned.log.likelihood.CS2,par=par))
  return(ll)
}



################################################################################
## 
## Log-prior, lognormal distributed
## ================================
##
## Calculates the logarithm of the multivariate probability density function of
## independent marginals:

FN.log.prior<- function(par, prior.mean, prior.sd) 
{
  
  ## Conversion of mean and standard deviations of the lognormal distributed 
  ## parameters theta and lambda into mean and standard deviations of 
  ## their normally distributed logarithm:

  prior.sdlog=sqrt(log(1+prior.sd^2/prior.mean^2))
  prior.meanlog=log(prior.mean)-prior.sdlog^2/2
  dlnorm.parameter.matrix=data.frame(par, prior.meanlog, prior.sdlog)
    
  FN.dlnorm=function(x)
  {
    y.log=dlnorm(x=x[1], meanlog=x[2], sdlog=x[3], log=T)
    return(y.log)
  }
    
  log.prior=0
  log.prior=log.prior+sum(apply(dlnorm.parameter.matrix, 1, FN.dlnorm))
  return(log.prior)
}


## Calculates the logarithm of the multivariate probability density function of
## independent marginals except of alpha and beta.

FN.log.prior.dep <- function(par, prior.mean, prior.sd, varcov) 
  {
  ## Conversion of mean and standard deviations of the lognormal distributed 
  ## parameters theta and lambda into mean and standard deviations of 
  ## their normally distributed logarithm:

    prior.sdlog=sqrt(log(1+prior.sd^2/prior.mean^2))
    prior.meanlog=log(prior.mean)-prior.sdlog^2/2
    dlnorm.parameter.matrix=data.frame(par[-(1:2)], prior.meanlog[-(1:2)], prior.sdlog[-(1:2)])
    
    FN.dlnorm=function(x)
    {
      y.log=dlnorm(x=x[1], meanlog=x[2], sdlog=x[3], log=T)
      return(y.log)
    }
    
    log.prior.dep=log(dmnorm(x=log(par[1:2]), mean=prior.meanlog[1:2], varcov, log=FALSE)/(par[1]*par[2]))
    log.prior=log.prior.dep+sum(apply(dlnorm.parameter.matrix, 1, FN.dlnorm))
    return(log.prior)
  }

## Faster version of FN.log.prior, used in sensitivity analysis
FN.log.prior.4SA <- function(par, prior.mean, prior.sd) 
{
  ## Conversion of mean and standard deviations of the lognormal distributed 
  ## parameters theta and lambda into mean and standard deviations of 
  ## their normally distributed logarithm:

  prior.sdlog=sqrt(log(1+prior.sd^2/prior.mean^2))
  prior.meanlog=log(prior.mean)-prior.sdlog^2/2
    
  log.prior=0
  for (i in 1:length(prior.mean))
  log.prior=log.prior+dlnorm(x=par[i], meanlog=prior.meanlog[i], sdlog=prior.sdlog[i], log=TRUE)
  return(log.prior)
}


################################################################################
## 
## Log-posterior
## =============
## Calculates the logarithm of the posterior probability density functions  
## (without normalization)

FN.log.posterior= function(par, prior.mean, prior.sd,...)
{
  
  log.posterior=FN.log.likelihood(par, data.set)+
                FN.log.prior(par, prior.mean, prior.sd)

  return(log.posterior)
}

# ## posterior function for only selected parameters to be varied
# FN.log.posterior= function(par, prior.mean, prior.sd, ...)
# # FN.log.posterior= function(par, prior.mean, prior.sd,...)  
# {
#  
#   variables=c(prior.mean[1:5],par)
#   log.posterior=FN.log.likelihood(par=variables, data.set)+
#   #log.posterior=FN.log.likelihood(data.set, par, ...)+
#     
#                 FN.log.prior(par, prior.mean=prior.mean[6], prior.sd=prior.sd[6])
#   #if (log.posterior==0) return(NA)
#   return(log.posterior)
# }


## Posterior function. Some parameter are fixed
FN.log.posterior.1CS.fixed= function(par, prior.mean, prior.sd, ...)  
{
  variables=c(par, prior.mean[4:6])
  log.posterior=FN.log.likelihood(par=variables, data.set)+
  #log.posterior=FN.log.likelihood(data.set, par, ...)+
  # prior.mean[...], ... parameters to be varied 
  FN.log.prior(par, prior.mean=prior.mean[c(1,2,3)], prior.sd=prior.sd[c(1,2,3)])
  #if (log.posterior==0) return(NA)
  return(log.posterior)
}

## Calculates the logarithm of the posterior probability density functions  
## (without normalization) used for datasets that contain one or two condition
## states per pipe

FN.log.posterior.2CS= function(par, prior.mean, prior.sd,...)
{
  
  log.posterior=FN.log.likelihood.2CS(par, data.set)+
    FN.log.prior(par, prior.mean, prior.sd)
  
  return(log.posterior)
}

## for selected parameters to be fixed. The variable par.fixed contains
## both the variable parameters and at least one fixed parameter to be 
## specified within the function
FN.log.posterior.2CS.fixed= function(par, prior.mean, prior.sd,...)
{
  par.fixed=c(par, prior.mean[4:6])
  log.posterior=FN.log.likelihood.2CS(par=par.fixed, data.set)+
    FN.log.prior(par, prior.mean=prior.mean[c(1,2,3)], 
                 prior.sd=prior.sd[c(1,2,3)])
  
  return(log.posterior)
}







