##############################################################################
##
## Likelihood of the sewer deterioration models SWIP-SDM
##
## Implemetation of probabilistic sewer deterioration model and likelihood-
## function developed by Egger et al. (2013):
##
## Egger, C., Scheidegger, A., Reichert, P. and Maurer, M. (2013) 
## Sewer deterioration modeling with condition data lacking historical records. 
## Water Research 47(17), 6762-6779.
##
##
## The model described in Egger et al. (2013) is extended so that two
## subsequently observed condiion states can be used
##
##
## -----------------------------------------------------------------------------
## Arguments:
## t:     Sewer pipe age
## theta: Vector of parameters of the distrubtions f_i of the retention times of
##        a sewer pipe whitin different condition states CS_i
## Tstern:Tao
##
##
## -----------------------------------------------------------------------------
## The deterioration model describes the times X_i=t_1-t_(i-1) a pipe spends in
## condition state CS_i.
##
## =============================================================================
##
## Content:
##
## 1.) Survival functions (these functions are also used for inference 
##     with one observed CS)
##
## 2.) Probabilities that a pipe is in CS i at age tau1 and in CS j>=i at
##     age tau2>tau1 (these probabilities are used to infer deteriotaion 
##     parameters from two subsequent observations of CS).
##
## 3.) Probabilities that a pipe has not been replaced before D=T_stern given 
##     the pipe was in condition states i=1...3 at age tau=T1 (these
##     probabilities are used to infer deteriotaion parameters from one or 
##     observations of CS as described in Egger et al. 2013).
##
## 4.) Proabilities that a pipe has not been replaced before tau2 given the
##     pipe was in CS i at age tau1 and in CS j>=i at age tau2>tau1 (these
##     probabilities are used to infer deteriotaion parameters from one or 
##     two subsequent observations of CS).
##
## 5.) Functions needed to forecast CS


################################################################################
##
## Survival functions
##
## ==================
##
## The time T_1 which elapses between construction of a pipe until the first
## transition of condition state is assumed to be Weibull-distributed with shape
## theta[1] and scale theta[2]
##
## T_i, i>1 are assumed to be exponential distributed with scales theta[i>=3]
##
## The time of construction t_0 is equal to zero
##
## Based of the pdfs f_i of the transition times T_i, survival functions S_i are
## calculated.
##
## -----------------------------------------------------------------------------



################################################################################
##
## 1.) Survival functions
##

## Function that calculates a factor included in some survival functions. Theta
## includes the the parameters shape and scale of the Weibull distribution and mju
## is the parameter of an exponential distribution:

FN.G=function(t, theta, mju)
{
  #print(c(t, theta, mju))
  integral.G=function(t1)
  {
    I=theta[1]/theta[2]*(t1/theta[2])^(theta[1]-1)*exp(-(t1/theta[2])^theta[1]-(t-t1)/mju)   
  }
  G=integrate(f=integral.G, lower=0.0001, upper=t)$value
  return(G)
}


## Survival function S1(t)=1-F1(t) with shape theta[1] and scale theta[2]

FN.S.1=function(t, theta)
{
  if  (any(theta[!is.na(theta)]<0)) return(Inf)
  S1=exp(-(t/theta[2])^theta[1])
  return(S1)
}


## Residence time in CS 1

FN.mean.t1=function(sample)
{
  mean.t1=sample[2]*gamma(1+1/sample[1])
  return(mean.t1)
}

## Survival function S_2(t) with shape theta[1], scale theta[2]
## and mju=theta[3]

FN.S.2=function(t, theta, S1)
{
  S2=S1+FN.G(t, theta, mju=theta[3])
  return(S2)
}

## Survival function S_3(t) with shape theta[1], scale theta[2]
## and mju=theta[3:4]

FN.S.3=function(t, theta, S1)
{  
  S3=S1-theta[3]/(theta[4]-theta[3])*
    FN.G(t, theta, mju=theta[3])+theta[4]/(theta[4]-theta[3])*
    FN.G(t, theta, mju=theta[4])
  return(S3)
}  

## Survival function S_4(t) with shape theta[1], scale theta[2]
## and mju=theta[3:5]

FN.S.4=function(t, theta, S1)
{
  S4=S1+theta[3]^2/((theta[4]-theta[3])*(theta[5]-theta[3]))*FN.G(t, theta, mju=theta[3])-
    theta[4]^2/((theta[5]-theta[4])*(theta[4]-theta[3]))*FN.G(t, theta, mju=theta[4])+
    theta[5]^2/((theta[5]-theta[4])*(theta[5]-theta[3]))*FN.G(t, theta, mju=theta[5])
  return(S4)
}

################################################################################
##
## 2.) Probabilities that a pipe is in CS i at age tau1 and in CS j>=i at
##     at age tau2>tau1
##  
## These probabilities are used to infer deteriotaion parameters from
## two subsequent observation of CS.
## 
## Probability that a pipe is in CS 1 at age tau1 and tau2, use FN.S.1

## General term used in different equations below:

FN.term=function(alpha, beta, t1)
{
  f=alpha/beta*(t1/beta)^(alpha-1)*exp(-(t1/beta)^alpha)
  return(f)
}

## (2) Probability that a pipe is in CS 1 at age tau1 and in CS 2 
## at age tau2:

FN.P.1.2=function(alpha, beta, mju, tau1, tau2)
{
  integral=function(t1)
  {
    I=FN.term(alpha, beta, t1)*exp(-(tau2-t1)/mju)
  }
  P.1.2=integrate(f=integral, lower=tau1, upper=tau2)$value
  return(P.1.2)
}



## (3) Probability that a pipe is in CS 1 at age tau1 and in CS 3
## at age tau2:

FN.P.1.3=function(alpha, beta, mju, tau1, tau2)
{
  integral=function(t1)
  {
    I=FN.term(alpha, beta, t1)*(1-exp(-(tau2-t1)/mju))
  }
  P.1.3=integrate(f=integral, lower=tau1, upper=tau2)$value
  return(P.1.3)
}

## (4) Probability that a pipe is in CS 2 at age tau1 and in CS 3
## at age tau2:

FN.P.2.3=function(alpha, beta, mju, tau1, tau2)
{
  integral=function(t1)
  {
    I=FN.term(alpha, beta, t1)*(-exp(-(tau2-t1)/mju)+
                                  exp(-(tau1-t1)/mju))
  }
  P.2.3=integrate(f=integral, lower=0, upper=tau1)$value
  return(P.2.3)
}

## (5) Probability that a pipe is in C2 2 at age tau1 and tau2:

FN.P.2.2=function(alpha, beta, mju, tau1, tau2)
{
  integral=function(t1)
  {
    I=FN.term(alpha, beta, t1)*(exp(-(tau2-t1)/mju))
                              
  }
  P.2.2=integrate(f=integral, lower=0, upper=tau1)$value
  return(P.2.2)
}

## (6) Probability that a pipe is in C2 3 at age tau1 and tau2:

FN.P.3.3=function(alpha, beta, mju, tau1, tau2)
{
  integral=function(t1)
  {
    I=FN.term(alpha, beta, t1)*(1-exp(-(tau1-t1)/mju))
    
  }
  P.3.3=integrate(f=integral, lower=0, upper=tau1)$value
  return(P.3.3)
}

################################################################################
##
## 3.) Probabilities that a pipe has not been replaced before D given the pipe 
##     was in condition state i=1...5 at age tau=T1
##
## These probabilities are used for deterioration parameter inference with
## one observed CS at age tau=T1 as described in Egger et al. (2013)
## =================


## Probabilities that a pipe has not been rehabilitated before age D=T_stern 
## given the pipe is in condition state i=1. Rehabilitation started first at the 
## age of R=TR.

FN.P.nr.CS1=function(lambda, T_stern, TR)
{
  P.nr.CS1=exp(-lambda[1]*(T_stern-TR))
  return(P.nr.CS1)
}

## Probabilities that a pipe has not been replaced before TD=T_stern given the 
## pipe was in condition state i=2 at age tau=T1. Rehabilitation started first 
## at the age of R=TR.

FN.P.nr.CS2=function(theta, lambda, T1, T_stern, TR)
{
  integral.P.nr.CS2.TR.num=function(t1)
  {
    I=exp(-lambda[2]*(T1-TR))*(t1)^(theta[1]-1)*
      exp(-(t1/theta[2])^theta[1])*exp(-(T1-t1)/theta[3])
    return(I)
  }
  
  integral.P.nr.CS2.T1.num=function(t1)
  {
    I=exp(-lambda[1]*(t1-TR)-lambda[2]*(T1-t1))*(t1)^(theta[1]-1)*
      exp(-(t1/theta[2])^theta[1])*exp(-(T1-t1)/theta[3])
    return(I)
  }
  
  integral.P.nr.CS2.nom=function(t1)
  {
    I=(t1)^(theta[1]-1)*exp(-(t1/theta[2])^theta[1]-(T1-t1)/theta[3])
  }
  
  P.nr.CS2=(integrate(f=integral.P.nr.CS2.TR.num, lower=0.0001, upper=TR)$value+
              integrate(f=integral.P.nr.CS2.T1.num, lower=TR, upper=T1)$value)/
    integrate(f=integral.P.nr.CS2.nom, lower=0.0001, upper=T1, subdivisions=1000L)$value*
    exp(-lambda[2]*(T_stern-T1))
  
  return(P.nr.CS2)
}


## Probabilities that a pipe has not been replaced before D=T_stern given 
## the pipe was in condition state i=3 at age tau=T1. Rehabilitation started first 
## at the age of R=TR.

FN.P.nr.CS3=function(theta, lambda, T1, T_stern, TR)
{
  const.1=((lambda[3]-lambda[2])*theta[3]-1)^(-1)
  
  ## numerator, Summand 1, t1, t2<TR
  integral.P.nr.CS3.num.1=function(t1)
  {
    I=exp(-lambda[3]*(T1-TR))*t1^(theta[1]-1)*exp(-(t1/theta[2])^theta[1])*
      (1-exp(-(TR-t1)/theta[3]))
    #if (I==Inf) return(0)                                             
    return(I)
  }
  
  ## numerator, Summand 2, t1<TR<=t2<=T1
  
  integral.P.nr.CS3.num.2=function(t1)
  {
    I=t1^(theta[1]-1)*exp(-(t1/theta[2])^theta[1])*
      (exp(-lambda[2]*(T1-TR)-(T1-t1)/theta[3])-
         exp(-(TR-t1)/theta[3]-lambda[3]*(T1-TR)))
    #if (I==Inf) return(0)                                             
    return(I)
  }
  
  ## numerator, Summand 3, TR<t1,t2<=T1
  
  integral.P.nr.CS3.num.3=function(t1)
  {
    I=t1^(theta[1]-1)*exp(-(t1/theta[2])^theta[1])*exp(-lambda[1]*(t1-TR))*
      (exp(-lambda[2]*(T1-t1)-(T1-t1)/theta[3])-
         exp(-lambda[3]*(T1-t1)))
    #if (I==Inf) return(0)                                             
    return(I)
  }
  
  ## nominator
  integral.P.nr.CS3.nom=function(t1)
  {
    I=t1^(theta[1]-1)*exp(-(t1/theta[2])^theta[1])*(1-exp(-(T1-t1)/theta[3]))
    #if (I==Inf) return(0)
    return(I)
  }
  #print(c(T1, theta, lambda, T1, T_stern, TR))
  P.nr.CS3=(integrate(f=integral.P.nr.CS3.num.1, lower=0.0001, upper=TR)$value+
              const.1*integrate(f=integral.P.nr.CS3.num.2, lower=0.0001, upper=TR)$value+
              const.1*integrate(f=integral.P.nr.CS3.num.3, lower=TR, upper=T1)$value)/
    (integrate(f=integral.P.nr.CS3.nom, lower=0.0001, upper=T1)$value)*
    exp(-lambda[3]*(T_stern-T1))
 
  
  return(P.nr.CS3)
}

## Probabilities that a pipe has not been replaced before T1 given the pipe is
## in condition state i=4

FN.P.nr.CS4=function(theta, lambda, T1, T_stern)
{
  integral.P.nr.CS4.num=function(t1)
  {
    I=theta[1]/theta[2]*(t1/theta[2])^(theta[1]-1)*exp(-(t1/theta[2])^theta[1])*
      exp(-lambda[1]*t1)*
      (((theta[4]*exp(-T1/theta[4]-lambda[3]*T1-T1/theta[3]-lambda[2]*T1)*
           (exp(T1/theta[4]+lambda[3]*T1+t1/theta[3]+t1*lambda[2])-exp(T1/theta[3]+lambda[2]*T1+t1/theta[4]+t1*lambda[3])))/
          ((theta[3]*lambda[3]*theta[4]-lambda[2]*theta[3]*theta[4]-theta[4]+theta[3])*(theta[4]*lambda[4]-lambda[3]*theta[4]-1)))-
         (exp(-lambda[4]*T1-T1/theta[3]-lambda[2]*T1)*(exp(lambda[4]*T1+t1/theta[3]+t1*lambda[2])-exp(T1/theta[3]+lambda[2]*T1+t1*lambda[4])))/
         ((theta[3]*lambda[4]-lambda[2]*theta[3]-1)*(theta[4]*lambda[4]-lambda[3]*theta[4]-1)))
    return(I)
  }
  ## nominator
  integral.P.nr.CS4.nom=function(t1)
  {
    I=theta[1]/theta[2]*(t1/theta[2])^(theta[1]-1)*exp(-(t1/theta[2])^theta[1])*
      (exp(-(T1-t1)/theta[3])*(-theta[4]/(theta[3]-theta[4])-1)-
         (-theta[4]/(theta[3]-theta[4])*exp(-(T1-t1)/theta[4])-1))
    return(I)
  }
  P.nr.CS4=integrate(f=integral.P.nr.CS4.num, lower=0, upper=T1)$value/
    integrate(f=integral.P.nr.CS4.nom, lower=0, upper=T1)$value*
    exp(-lambda[4]*(T_stern-T1))
  return(P.nr.CS4)
}



# ## Probabilities that a pipe has not been replaced before T1 given the pipe is
# ## in condition state i=5
# 
# FN.P.nr.CS5=function(theta, lambda, T1)
#   {
#    
#   integral.P.nr.CS5.num=function(t1)
#     {
#     ## some factors denoted by "numerator N.i"
# #     N.3=(theta[4]*lambda[4]*theta[5]-lambda[3]*theta[4]*theta[5]-theta[5]+theta[4])*
# #         (theta[5]*lambda[5]*-theta[5]*lambda[4]-1)
# #     N.4=(theta[4]*lambda[5]-theta[4]*lambda[3]-1)*(theta[5]*lambda[5]-lambda[4]*theta[5]-1)
# #     N.5=lambda[2]*theta[3]+1
# #     
# #     summand.1=((theta[5]*(exp(T1/theta[3]+lambda[2]*T1)-exp(t1/theta[3]+t1*lambda[2]))*
# #               exp(-T1/theta[5]-lambda[4]*T1-T1/theta[4]-lambda[3]*T1-T1/theta[3]-lambda[2]*T1)*
# #               (exp(T1/theta[5]+lambda[4]*T1+t1/theta[4]+t1*lambda[3])-exp(T1/theta[4]+lambda[3]*T1+t1/theta[5]+t1*lambda[4])))/
# #               N.5)/N.3
# # 
# #     summand.2=(((exp(T1/theta[3]+lambda[2]*T1)-exp(t1/theta[3]+t1*lambda[2]))*
# #               exp(-lambda[5]*T1-T1/theta[4]-lambda[3]*T1-T1/theta[3]-lambda[2]*T1)*
# #               (exp(lambda[5]*T1+t1/theta[4]+t1*lambda[3])-exp(T1/theta[4]+lambda[3]*T1+t1*lambda[5])))/
# #               N.5)/N.4
# #  
# #     multiplicator.1=theta[1]/theta[2]*(t1/theta[2])^(theta[1]-1)*exp(-(t1/theta[2])^theta[1])*exp(-lambda[1]*t1)
# #   
# #     
# #     I= multiplicator.1*(summand.1-summand.2)
# #          
#     
# #     N.1=((lambda[3]*theta[4]+1)*(lambda[4]*theta[5]+1)*lambda[5])
# #     N.2=(theta[4]*lambda[4]*theta[5]-lambda[3]*theta[4]*theta[5]-theta[5]+theta[4])*
# #         (theta[4]*lambda[5]-lambda[3]*theta[4]-1)*(theta[5]*lambda[5]-lambda[4]*theta[5]-1)
# #     
# #     I=theta[1]/theta[2]*(t1/theta[2])^(theta[1]-1)*exp(-(t1/theta[2])^theta[1])*exp(-lambda[1]*t1)/(N.1*N.2)*
# #       ((-theta[4]^2*lambda[4]*theta[5]^2*lambda[5]^2-theta[4]^2*theta[5]*lambda[5]^2+theta[4]^2*lambda[4]^2*theta[5]^2*lambda[5]+
# #       2*theta[4]^2*lambda[4]*theta[5]*lambda[5]+theta[4]^2*lambda[5])*
# #       exp((-T1-t1)/theta[4]-lambda[3]*(T1-t1))+
# #       (lambda[3]*theta[4]^2*theta[5]^2*lambda[5]^2+theta[4]*theta[5]^2*lambda[5]^2-lambda[3]^2*theta[4]^2*theta[5]^2*lambda[5]-
# #       2*lambda[3]*theta[4]*theta[5]^2*lambda[5]-theta[5]^2*lambda[5])*
# #       exp((-T1-t1)/theta[5]-lambda[4]*(T1-t1))+
# #       (-lambda[3]*theta[4]^2*lambda[4]^2*theta[5]^2-theta[4]*lambda[4]^2*theta[5]^2+lambda[3]^2*theta[4]^2*lambda[4]*theta[5]^2+
# #       2*lambda[3]*theta[4]*lambda[4]*theta[5]^2+lambda[4]*theta[5]^2-lambda[3]*theta[4]^2*lambda[4]*theta[5]-
# #       theta[4]*lambda[4]*theta[5]+lambda[3]^2*theta[4]^2*theta[5]+2*lambda[3]*theta[4]*theta[5]+theta[5])*
# #       exp(-lambda[5]*(T1-t1)))
#     
#     N.3=(theta[4]*lambda[4]*theta[5]-lambda[3]*theta[4]*theta[5]-theta[5]+theta[4])*
#         (theta[5]*lambda[5]*-theta[5]*lambda[4]-1)
#     N.4=(theta[4]*lambda[5]-theta[4]*lambda[3]-1)*(theta[5]*lambda[5]-lambda[4]*theta[5]-1)
#     N.5=(theta[3]*lambda[3]*theta[4]-lambda[2]*theta[3]*theta[4]-theta[4]+theta[3])*
#         (theta[3]*lambda[4]*theta[5]-lambda[2]*theta[3]*theta[5]-theta[5]+theta[3])
#     N.6=(theta[3]*lambda[3]*theta[4]-lambda[2]*theta[3]*theta[4]-theta[4]+theta[3])*
#         (theta[3]*lambda[5]-lambda[2]*theta[3]-1)
#  
#       I <- theta[1]/theta[2]*(t1/theta[2])^(theta[1]-1)*exp(-(t1/theta[2])^theta[1])*exp(-lambda[1]*t1)*
#          (
#            theta[5]/(N.3*N.5)*
#            (
#              (theta[3]*theta[4]*lambda[4]*theta[5]-theta[3]*lambda[3]*theta[4]*theta[5]-theta[3]*theta[5]+theta[3]*theta[4])*
#              exp(-lambda[2]*(T1-t1)-(T1-t1)/theta[3])-
#              (theta[3]*theta[4]*lambda[4]*theta[5]-lambda[2]*theta[3]*theta[4]*theta[5]-theta[4]*theta[5]+theta[3]*theta[4])*
#              exp(-lambda[3]*(T1-t1)-(T1-t1)/theta[4])+
#              (theta[3]*lambda[3]*theta[4]*theta[5]-lambda[2]*theta[3]*theta[4]*theta[5]-theta[4]*theta[5]+theta[3]*theta[5])*
#              exp(-lambda[4]*(T1-t1)-(T1-t1)/theta[5])
#            )-
#            (1/(N.4*N.6)*
#            (
#              (theta[3]*theta[4]*lambda[5]-theta[3]*lambda[3]*theta[4]-theta[3])*
#              exp(-lambda[2]*(T1-t1)-(T1-t1)/theta[3])-
#              (theta[3]*theta[4]*lambda[5]-lambda[2]*theta[3]*theta[4]-theta[4])*
#              exp(-lambda[3]*(T1-t1)-(T1-t1)/theta[4])+
#              (theta[3]*lambda[3]*theta[4]-lambda[2]*theta[3]*theta[4]-theta[4]+theta[3])*
#              exp(-lambda[5]*(T1-t1))
#            )
#            )
#          )
#          
#      
#       return(I)
#     }
# 
#   ## nominator
#   integral.P.nr.CS5.nom=function(t1)
#     {
#         multiplicator.2=theta[1]/theta[2]*(t1/theta[2])^(theta[1]-1)*exp(-(t1/theta[2])^theta[1])
#         
#         I= multiplicator.2*
#            (exp(-T1/theta[5]-T1/theta[4]-T1/theta[3])*
#            (theta[4]*theta[5]^2*exp(T1/theta[5]+T1/theta[4]+T1/theta[3])-
#            theta[3]*theta[5]^2*exp(T1/theta[5]+T1/theta[4]+T1/theta[3])-
#            theta[4]^2*theta[5]*exp(T1/theta[5]+T1/theta[4]+T1/theta[3])+
#            theta[3]^2*theta[5]*exp(T1/theta[5]+T1/theta[4]+T1/theta[3])+
#            theta[3]*theta[4]^2*exp(T1/theta[5]+T1/theta[4]+T1/theta[3])-
#            theta[3]^2*theta[4]*exp(T1/theta[5]+T1/theta[4]+T1/theta[3])-
#            theta[3]^2*theta[5]*exp(T1/theta[5]+T1/theta[4]+t1/theta[3])+
#            theta[3]^2*theta[4]*exp(T1/theta[5]+T1/theta[4]+t1/theta[3])+
#            theta[4]^2*theta[5]*exp(T1/theta[5]+T1/theta[3]+t1/theta[4])-
#            theta[3]*theta[4]^2*exp(T1/theta[5]+T1/theta[3]+t1/theta[4])-
#            theta[4]*theta[5]^2*exp(T1/theta[4]+T1/theta[3]+t1/theta[5])+
#            theta[3]*theta[5]^2*exp(T1/theta[4]+T1/theta[3]+t1/theta[5])))/
#            ((theta[4]-theta[3])*(theta[5]-theta[3])*(theta[5]-theta[4]))
# 
#       
#       
#       
# #       I=theta[1]/theta[2]*(t1/theta[2])^(theta[1]-1)*exp(-(t1/theta[2])^theta[1])*
# #         ((theta[4]*theta[5]^2-theta[3]*theta[5]^2-theta[4]^2*theta[5]+theta[3]^2*theta[5]+theta[3]*theta[4]^2-theta[3]^2*theta[4]-
# #           (theta[3]^2*theta[5]-theta[3]^2*theta[4])*exp(-(T1-t1)/theta[3])+
# #           (theta[4]^2*theta[5]-theta[3]*theta[4]^2)*exp(-(T1-t1)/theta[4])-
# #           (theta[4]*theta[5]^2-theta[3]*theta[5]^2)*exp(-(T1-t1)/theta[5]))/
# #           ((theta[4]-theta[3])*(theta[5]-theta[3])*(theta[5]-theta[4])))
#       return(I)
#     }  
#     
#   P.nr.CS5=integrate(f=integral.P.nr.CS5.num, lower=0, upper=T1)$value/
#             integrate(f=integral.P.nr.CS5.nom, lower=0, upper=T1)$value
#   return(P.nr.CS5)
#   }

# FN.P.nr.CS5=function(theta, lambda, T1)
#   {
# 
#   integral.P.nr.CS5.num=function(t1)
#     {
#       I <- theta[1]/theta[2]*(t1/theta[2])^(theta[1]-1)*exp(-(t1/theta[2])^theta[1])*
#       exp(-lambda[1]*t1)*
#       (exp((-lambda[2]-1/theta[3])*(T1-t1))-1)/(-lambda[2]*theta[3]-1)*
#       (theta[5]*exp(-T1/theta[5]-lambda[4]*T1-T1/theta[4]-lambda[3]*T1)*
#       (exp(T1/theta[5]+lambda[4]*T1 + t1/theta[4]+t1*lambda[3])-exp(T1/theta[4]+lambda[3]*T1+t1/theta[5]+t1*lambda[4]))/
#       ((theta[4]*lambda[4]*theta[5]-lambda[3]*theta[4]*theta[5]-theta[5]+theta[4])*(theta[5]*lambda[5]-lambda[4]*theta[5]-1))-
#       exp(-lambda[5]*T1-T1/theta[4]-lambda[3]*T1)*
#       (exp(lambda[5]*T1+t1/theta[4]+t1*lambda[3])-exp(T1/theta[4]+lambda[3]*T1+t1*lambda[5]))/
#       ((theta[4]*lambda[5]-lambda[3]*theta[4]-1)*(theta[5]*lambda[5]-lambda[4]*theta[5]-1)))
#       return(I)
#     }
#   ## nominator
#   integral.P.nr.CS5.nom=function(t1)
#     {
#       I=theta[1]/theta[2]*(t1/theta[2])^(theta[1]-1)*exp(-(t1/theta[2])^theta[1])*
#         exp(-(T1-t1)/theta[3])*
#         (-theta[4]^2/((theta[4]-theta[5])*(theta[3]-theta[4]))+theta[5]^2/((theta[5]-theta[3])*(theta[5]-theta[4]))-1)-
#         (-theta[4]^2/((theta[4]-theta[5])*(theta[3]-theta[4]))*exp(-(T1-t1)/theta[4])+
#           theta[5]^2/((theta[5]-theta[3])*(theta[5]-theta[4]))*exp(-(T1-t1)/theta[5])-1)
#       return(I)
#     }  
#     
#   P.nr.CS5=integrate(f=integral.P.nr.CS5.num, lower=0, upper=T1)$value/
#             integrate(f=integral.P.nr.CS5.nom, lower=0, upper=T1)$value
#   return(P.nr.CS5)
#   }



# library(cubature)
# 
# FN.P.nr.CS5=function(theta, lambda, T1)
#   {
# 
#   integral.P.nr.CS5.num=function(t)
#     {
#       if(t[2]<t[1]) return(0)
#       I <- theta[1]/theta[2]*(t[1]/theta[2])^(theta[1]-1)*exp(-(t[1]/theta[2])^theta[1])*exp(-lambda[1]*t[1])*
#         exp(-*(t[2]-t[1]))*1/theta[3]*exp(-(t[2]-t[1])/theta[3])*
#       (theta[5]*exp(-T1/theta[5]-lambda[4]*T1-T1/theta[4]-lambda[3]*T1)*
#       (exp(T1/theta[5]+lambda[4]*T1 + t[1]/theta[4]+t[1]*lambda[3])-exp(T1/theta[4]+lambda[3]*T1+t[1]/theta[5]+t[1]*lambda[4]))/
#       ((theta[4]*lambda[4]*theta[5]-lambda[3]*theta[4]*theta[5]-theta[5]+theta[4])*(theta[5]*lambda[5]-lambda[4]*theta[5]-1))-
#       exp(-lambda[5]*T1-T1/theta[4]-lambda[3]*T1)*
#       (exp(lambda[5]*T1+t[1]/theta[4]+t[1]*lambda[3])-exp(T1/theta[4]+lambda[3]*T1+t[1]*lambda[5]))/
#       ((theta[4]*lambda[5]-lambda[3]*theta[4]-1)*(theta[5]*lambda[5]-lambda[4]*theta[5]-1)))
#       return(I)
#     }
#   ## nominator
#   integral.P.nr.CS5.nom=function(t1)
#     {
#       I=theta[1]/theta[2]*(t1/theta[2])^(theta[1]-1)*exp(-(t1/theta[2])^theta[1])*
#         (exp(-(T1-t1)/theta[3])*
#         (-theta[4]^2/((theta[4]-theta[5])*(theta[3]-theta[4]))+theta[5]^2/((theta[5]-theta[3])*(theta[5]-theta[4]))-1)-
#         (-theta[4]^2/((theta[4]-theta[5])*(theta[3]-theta[4]))*exp(-(T1-t1)/theta[4])+
#           theta[5]^2/((theta[5]-theta[3])*(theta[5]-theta[4]))*exp(-(T1-t1)/theta[5])-1))
#       return(I)
#     }  
#     
#   P.nr.CS5=adaptIntegrate(f=integral.P.nr.CS5.num, lowerLimit=c(0, 0), upperLimit=c(T1, T1),tol=1e-4)$integral/
#             integrate(f=integral.P.nr.CS5.nom, lower=0, upper=T1)$value
#   return(P.nr.CS5)
#   }

# T1=20
# p=0.02
# lambda=rep(-log(1-p),5)
# # # lambda=c(0.001, 0.002, 0.01, 0.05, 0.1)
# # # # #lambda=c(0.01, 0.02, 0.05, 0.08, 0.2)
# theta=c(1.5,30,15,20,22)
# # # # #
# test1=FN.P.nr.CS1(lambda, T1)
# test2=FN.P.nr.CS2(theta, lambda, T1)
# test3=FN.P.nr.CS3(theta, lambda, T1)
# test3
# test4=FN.P.nr.CS4(theta, lambda, T1)
# test4
# test5=FN.P.nr.CS5(theta, lambda, T1)
# test5
# erg=c(test1, test2, test3, test4, test5)
# erg
# # 
# # 
# # test_11=integral.P.nr.CS5.num
# 
# ## test of numerators nd denominators
# P.nr.CS2.num=integrate(f=integral.P.nr.CS2, lower=0, upper=T1)$value
# P.nr.CS2.denom=(1-exp(-(T1/theta[2])^theta[1]))
# P.nr.CS2.num
# P.nr.CS2.denom
# 
# 
# P.nr.CS3.num=integrate(f=integral.P.nr.CS3.num, lower=0, upper=T1)$value
# P.nr.CS3.denom=integrate(f=integral.P.nr.CS3.nom, lower=0, upper=T1)$value
# P.nr.CS3.num
# P.nr.CS3.denom
# 
# P.nr.CS4.num=integrate(f=integral.P.nr.CS4.num, lower=0, upper=T1)$value
# P.nr.CS4.denom=integrate(f=integral.P.nr.CS4.nom, lower=0, upper=T1)$value
# P.nr.CS4.num
# P.nr.CS4.denom
# 
# P.nr.CS5.num=integrate(f=integral.P.nr.CS5.num, lower=0, upper=T1)$value
# P.nr.CS5.denom=integrate(f=integral.P.nr.CS5.nom, lower=0, upper=T1)$value
# P.nr.CS5.num
# P.nr.CS5.denom
# 
# 
# P.nr.CS2.num/P.nr.CS3.num
# P.nr.CS3.num/P.nr.CS4.num
# P.nr.CS4.num/P.nr.CS5.num
# 
# P.nr.CS2.denom/P.nr.CS3.denom
# P.nr.CS3.denom/P.nr.CS4.denom
# P.nr.CS4.denom/P.nr.CS5.denom

#?Vectorize
#ff=function(x) x-sum(x)
#ff(1:3)
#
#ff=function(x) x[1]
#ff(1:3)
#
#fff= Vectorize(ff)
#
#fff= Vectorize(ff)
#fff(1:4)
#
#x=1
#
#
#1/c*exp(-(x-z)/c)*(exp(-(Z-x)/d)*(-e/(d-e)-1)-(-e/(d-e)*exp(-(Z-x)/e)-1))
#1/c*exp(-(x-z)/c)*(exp(-(Z-x)/d)*(-f/(d-f)-1)-(-f/(d-f)*exp(-(Z-x)/f)-1))
#1/c*exp(-(x-t)/c)*(exp(-(Z-x)/d)*(-f/(d-f)-1)-(-f/(d-f)*exp(-(Z-x)/f)-1))


################################################################################
## 
##
## Function to calculate the probability vector of a pipe built at time
## zero being in conditions states i at time tau given the probability
## vector of the same pipe being in condition states i at time T1<tau
## 
## Arguments:
##
## theta: Parameter vector of survival functions
## t: point of time
## delta: time step used to approximate transition probabilities
## T1: Time when CS was observed
## tau: Time at which probability vector shall be forecasted
## ===================================================================
##
##


# ################################################################################
# ## Probabilities that a pipe has not been replaced before t.stern given the pipe 
# ## was in condition state i=1...5 at time T1
# ## =================
# 
# ## Probabilities that a pipe has not been replaced before T1 given the pipe is
# ## in condition state i=1
# 
# FN.P.nr.CS1=function(lambda, T1)
#   {
#   P.nr.CS1=exp(-lambda[1]*(T1))
#   return(P.nr.CS1)
#   }
# 
# ## Probabilities that a pipe has not been replaced before T1 given the pipe is
# ## in condition state i=2
# 
# FN.P.nr.CS2=function(theta, lambda, T1)
#   {
#   integral.P.nr.CS2=function(t1)
#     {
#     I=exp(-lambda[1]*t1-lambda[2]*(T1-t1))*theta[1]/theta[2]*(t1/theta[2])^(theta[1]-1)*
#       exp(-(t1/theta[2])^theta[1])
#     return(I)
#     }
#   P.nr.CS2=integrate(f=integral.P.nr.CS2, lower=0, upper=T1)$value/(1-exp(-(T1/theta[2])^theta[1]))
#   return(P.nr.CS2)
# }
# 
# ## Probabilities that a pipe has not been replaced before t.stern given the pipe is
# ## in condition state i=3 at time T1
# 
# FN.P.nr.CS3=function(theta, lambda, T1, t.stern)
#   {
#   ## numerator
#   integral.P.nr.CS3.num=function(t1)
#     {
#     I=theta[1]/theta[2]*(t1/theta[2])^(theta[1]-1)*exp(-(t1/theta[2])^theta[1])*
#     exp(-lambda[1]*t1)*
#     ((exp(-lambda[3]*(t.stern-T1)-(T1-t1)/theta[3]-lambda[2]*(T1-t1))-
#     exp(-lambda[3]*(t.stern-t1)))/(theta[3]*(lambda[3]-lambda[2])-1))
#     #if (I==Inf) return(0)
#     return(I)
#     }
#   ## nominator
#   integral.P.nr.CS3.nom=function(t1)
#     {
#     I=theta[1]/theta[2]*(t1/theta[2])^(theta[1]-1)*exp(-(t1/theta[2])^theta[1])*(1-exp(-(T1-t1)/theta[3]))
#     #if (I==Inf) return(0)
#     return(I)
#     }
#   
#   P.nr.CS3=integrate(f=integral.P.nr.CS3.num, lower=0, upper=T1)$value/
#            integrate(f=integral.P.nr.CS3.nom, lower=0, upper=T1)$value
#   return(P.nr.CS3)
#   }

###########################################################################
##
## 4.) Proabilities that a pipe has not been replaced before tau2 given the
##     pipe was in CS i at age tau1 and in CS j>=i at age tau2>tau1 
##
## These probabilities are used to infer deteriotaion parameters from two 
## subsequent observations of CS.
##
## theta.det is a part of the parameter vector exclusively including the
## deterioration parameters alpha, beta, and mju

## There need to be consiered 13 cases:

## (1) R>=tau2, this probability is one. 

## (2) tau1<=R<=tau2, C(tau1)=C(tau2)=1, use function FN.P.nr.CS1

## (3) R<=tau1<=tau2, C(tau1)=C(tau2)=1, use function FN.P.nr.CS1

## (4) tau1<=R<=tau2, C(tau1)=1,C(tau2)=2

FN.P.nr.case.4=function(theta.det, lambda, tau1, tau2, D, R)
{
  integral.P.nr.case.4.R.num=function(t1)
  {
    I=exp(-lambda[2]*(tau2-R))*(t1)^(theta.det[1]-1)*
      exp(-(t1/theta.det[2])^theta.det[1])*exp(-(tau2-t1)/theta.det[3])
    return(I)
  }
  
  integral.P.nr.case.4.tau2.num=function(t1)
  {
    I=exp(-lambda[1]*(t1-R)-lambda[2]*(tau2-t1))*(t1)^(theta.det[1]-1)*
      exp(-(t1/theta.det[2])^theta.det[1])*exp(-(tau2-t1)/theta.det[3])
    return(I)
  }
  
  integral.P.nr.case.4.nom=function(t1)
  {
    I=(t1)^(theta.det[1]-1)*exp(-(t1/theta.det[2])^theta.det[1]-(tau2-t1)/theta.det[3])
  }
  
  P.nr.case.4=(integrate(f=integral.P.nr.case.4.R.num, lower=tau1, upper=R)$value+
              integrate(f=integral.P.nr.case.4.tau2.num, lower=R, upper=tau2)$value)/
    integrate(f=integral.P.nr.case.4.nom, lower=tau1, upper=tau2, subdivisions=1000L)$value*
    exp(-lambda[2]*(D-tau2))
  
  return(P.nr.case.4)
}

## (5) R<=tau1<=tau2, C(tau1)=1,C(tau2)=2

FN.P.nr.case.5=function(theta.det, lambda, tau1, tau2, D, R)
{
  integral.P.nr.case.5.num=function(t1)
  {
    I=exp(-lambda[1]*(t1-tau1)-lambda[2]*(tau2-t1))*
      FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      exp(-(tau2-t1)/theta.det[3])
    return(I)
  }
  
  integral.P.nr.case.5.nom=function(t1)
  {
    I=FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      exp(-(tau2-t1)/theta.det[3])
  }
  
  P.nr.case.5=exp(-lambda[1]*(tau1-R))*
      integrate(f=integral.P.nr.case.5.num, lower=tau1, upper=tau2)$value/
    integrate(f=integral.P.nr.case.5.nom, lower=tau1, upper=tau2, subdivisions=1000L)$value*
    exp(-lambda[2]*(D-tau2))
  
  return(P.nr.case.5)
}

## (6) tau1<=R<=tau2, C(tau1)=1,C(tau2)=3

FN.P.nr.case.6=function(theta.det, lambda, tau1, tau2, D, R)
{
  const.1=((lambda[3]-lambda[2])*theta.det[3]-1)^(-1)
  
  ## numerator, Summand 1, t1, t2<R
  integral.P.nr.case.6.num.1=function(t1)
  {
    I=exp(-lambda[3]*(tau2-R))*t1^(theta.det[1]-1)*exp(-(t1/theta.det[2])^theta.det[1])*
      (1-exp(-(R-t1)/theta.det[3]))
    #if (I==Inf) return(0)                                             
    return(I)
  }
  
  ## numerator, Summand 2, t1<R<=t2<=tau2
  
  integral.P.nr.case.6.num.2=function(t1)
  {
    I=t1^(theta.det[1]-1)*exp(-(t1/theta.det[2])^theta.det[1])*
      (exp(-lambda[2]*(tau2-R)-(tau2-t1)/theta.det[3])-
         exp(-(R-t1)/theta.det[3]-lambda[3]*(tau2-R)))
    #if (I==Inf) return(0)                                             
    return(I)
  }
  
  ## numerator, Summand 3, R<t1,t2<=tau2
  
  integral.P.nr.case.6.num.3=function(t1)
  {
    I=t1^(theta.det[1]-1)*exp(-(t1/theta.det[2])^theta.det[1])*exp(-lambda[1]*(t1-R))*
      (exp(-lambda[2]*(tau2-t1)-(tau2-t1)/theta.det[3])-
         exp(-lambda[3]*(tau2-t1)))
    #if (I==Inf) return(0)                                             
    return(I)
  }
  
  ## nominator
  integral.P.nr.case.6.nom=function(t1)
  {
    I=t1^(theta.det[1]-1)*exp(-(t1/theta.det[2])^theta.det[1])*(1-exp(-(tau2-t1)/theta.det[3]))
    #if (I==Inf) return(0)
    return(I)
  }
  
  P.nr.case.6=(integrate(f=integral.P.nr.case.6.num.1, lower=tau1, upper=R)$value+
              const.1*integrate(f=integral.P.nr.case.6.num.2, lower=tau1, upper=R)$value+
              const.1*integrate(f=integral.P.nr.case.6.num.3, lower=R, upper=tau2)$value)/
    (integrate(f=integral.P.nr.case.6.nom, lower=tau1, upper=tau2)$value)*
    exp(-lambda[3]*(D-tau2))
  
  
  return(P.nr.case.6)
}

## (7) R<=tau1<=tau2, C(tau1)=1,C(tau2)=3

FN.P.nr.case.7=function(theta.det, lambda, tau1, tau2, D, R)
{
  integral.P.nr.case.7.num=function(t1)
  {
    I=exp(-lambda[1]*(t1-tau1))*
      FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      (exp(-lambda[2]*(tau2-t1)-(tau2-t1)/theta.det[3])-exp(-lambda[3]*(tau2-t1)))/
      ((lambda[3]-lambda[2])*theta.det[3]-1)
    return(I)
  }
  
  integral.P.nr.case.7.nom=function(t1)
  {
    I=FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      (1-exp(-(tau2-t1)/theta.det[3]))
  }
  
  P.nr.case.7=exp(-lambda[1]*(tau1-R))*
    integrate(f=integral.P.nr.case.7.num, lower=tau1, upper=tau2)$value/
    integrate(f=integral.P.nr.case.7.nom, lower=tau1, upper=tau2, subdivisions=1000L)$value*
    exp(-lambda[3]*(D-tau2))
  
  return(P.nr.case.7)
}

## (8) tau1<=R<=tau2, C(tau1)=2,C(tau2)=2

FN.P.nr.case.8=function(lambda,D, R)
{
  P.nr.case.8=exp(-lambda[2]*(D-R))
  return(P.nr.case.8)
}

## (9) R<=tau1<=tau2, C(tau1)=2,C(tau2)=2

FN.P.nr.case.9=function(theta.det, lambda, tau1, tau2, D, R)
{
  integral.P.nr.case.9.num.1=function(t1)
  {
    I=FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      exp(-lambda[2]*(tau2-R)-(tau2-t1)/theta.det[3])
  }
  
  integral.P.nr.case.9.num.2=function(t1)
  {
    I=FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      exp(-lambda[1]*(t1-R)-lambda[2]*(tau2-t1)-(tau2-t1)/theta.det[3])
  }
  
  integral.P.nr.case.9.nom=function(t1)
  {
    I=FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      exp(-(tau2-t1)/theta.det[3])
  }
  
  P.nr.case.9=
    (integrate(f=integral.P.nr.case.9.num.1, lower=0, upper=R)$value+
       integrate(f=integral.P.nr.case.9.num.2, lower=R, upper=tau1)$value)/
    integrate(f=integral.P.nr.case.9.nom, lower=0, upper=tau1, subdivisions=1000L)$value*
    exp(-lambda[2]*(D-tau2))
  
  return(P.nr.case.9)
}

## (10) tau1<=T<=tau2, C(tau1)=2,C(tau2)=3

FN.P.nr.case.10=function(theta.det, lambda, tau1, tau2, D, R)
{
  integral.P.nr.case.10.num.1=function(t1)
  {
    I=FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      exp(-lambda[3]*(tau2-R))*
            (exp(-(tau1-t1)/theta.det[3])-exp(-(R-t1)/theta.det[3]))
  }
   
  integral.P.nr.case.10.num.2=function(t1)
  {
    I=FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      (exp(-lambda[2]*(tau2-R)-(tau2-t1)/theta.det[3])-
         exp(-lambda[3]*(tau2-R)-(R-t1)/theta.det[3]))/
      ((lambda[3]-lambda[2])*theta.det[3]-1)
      
  }
  
  integral.P.nr.case.10.nom=function(t1)
  {
    I=FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      (exp(-(tau1-t1)/theta.det[3])-exp(-(tau2-t1)/theta.det[3]))
  }
  
  P.nr.case.10=
    (integrate(f=integral.P.nr.case.10.num.1, lower=0, upper=tau1)$value+
       integrate(f=integral.P.nr.case.10.num.2, lower=0, upper=tau1)$value)/
    integrate(f=integral.P.nr.case.10.nom, lower=0, upper=tau1, subdivisions=1000L)$value*
    exp(-lambda[3]*(D-tau2))
  
  return(P.nr.case.10) 
}

## (11) R<=tau1<=tau2, C(tau1)=2,C(tau2)=3

FN.P.nr.case.11=function(theta.det, lambda, tau1, tau2, D, R)
{
  integral.P.nr.case.11.num.1=function(t1)
  {
    I=FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      (exp(-lambda[2]*(tau2-R)-(tau2-t1)/theta.det[3])-
          exp(-lambda[2]*(tau1-R)-lambda[3]*(tau2-tau1)-(tau1-t1)/theta.det[3]))/
      ((lambda[3]-lambda[2])*theta.det[3]-1)
  }
  
  integral.P.nr.case.11.num.2=function(t1)
  {
    I=FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      exp(-lambda[1]*(t1-R))*(exp(-lambda[2]*(tau2-t1)-(tau2-t1)/theta.det[3])-
                  exp(-lambda[2]*(tau1-t1)-lambda[3]*(tau2-tau1)-(tau1-t1)/theta.det[3]))/
      ((lambda[3]-lambda[2])*theta.det[3]-1)
  }
  
  integral.P.nr.case.11.nom=function(t1)
  {
    I=FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      (exp(-(tau1-t1)/theta.det[3])-exp(-(tau2-t1)/theta.det[3]))
  }
  
  P.nr.case.11=
    (integrate(f=integral.P.nr.case.11.num.1, lower=0, upper=R)$value+
       integrate(f=integral.P.nr.case.11.num.2, lower=R, upper=tau1)$value)/
    integrate(f=integral.P.nr.case.11.nom, lower=0, upper=tau1, subdivisions=1000L)$value*
    exp(-lambda[3]*(D-tau2))
  
  return(P.nr.case.11) 
}

## (12) tau1<=R<=tau2, C(tau1)=3,C(tau2)=3

FN.P.nr.case.12=function(lambda,D, R)
{
  P.nr.case.12=exp(-lambda[3]*(D-R))
  return(P.nr.case.12)
}

## (13) R<=tau1<=tau2, C(tau1)=3,C(tau2)=3

FN.P.nr.case.13=function(theta.det, lambda, tau1, tau2, D, R)
{
  integral.P.nr.case.13.num.1=function(t1)
  {
    I=FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      exp(-lambda[3]*(tau2-R))*
      (1-exp(-(R-t1)/theta.det[3]))
  }
  
  integral.P.nr.case.13.num.2=function(t1)
  {
    I=FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      (exp(-lambda[2]*(tau1-R)-lambda[3]*(tau2-tau1)-(tau1-t1)/theta.det[3])-
         exp(-lambda[3]*(tau2-R)-(R-t1)/theta.det[3]))/
      ((lambda[3]-lambda[2])*theta.det[3]-1)
  }
  
  integral.P.nr.case.13.num.3=function(t1)
  {
    I=FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      exp(-lambda[1]*(t1-R))*
      (exp(-lambda[2]*(tau1-t1)-lambda[3]*(tau2-tau1)-(tau1-t1)/theta.det[3])-exp(-lambda[3]*(tau2-t1)))/
      ((lambda[3]-lambda[2])*theta.det[3]-1)
  }
  
  integral.P.nr.case.13.nom=function(t1)
  {
    I=FN.term(alpha=theta.det[1], beta=theta.det[2], t1)*
      (1-exp(-(tau1-t1)/theta.det[3]))
  }
      
  P.nr.case.13=
    (integrate(f=integral.P.nr.case.13.num.1, lower=0, upper=R)$value+
       integrate(f=integral.P.nr.case.13.num.2, lower=0, upper=R)$value+
    integrate(f=integral.P.nr.case.13.num.3, lower=R, upper=tau1)$value)/
    integrate(f=integral.P.nr.case.13.nom, lower=0, upper=tau1, subdivisions=1000L)$value*
    exp(-lambda[3]*(D-tau2))
  
  return(P.nr.case.13) 
}


###########################################################################
##
## 5.) Functions needed to forecast CS
##
##
## Function to calculate the probability vector that a pipe is in 
## condition state i at age t derived from given survival functions

FN.Pi.t=function(t, theta)
{
  Pi.t=rep(0, length(theta))
  Pi.t[1]=FN.S.1(t, theta)
  Pi.t[2]=FN.S.2(t, theta, Pi.t[1])-Pi.t[1]
  if (length(theta==3))
    Pi.t[3]=1-FN.S.2(t, theta, Pi.t[1])
  return(Pi.t)
}

##==============================================================================

## Function which calculates the transition probabilities that a pipe changes 
## from condition state i to i+1 within the time step delta at time t.

FN.p.iiplus1=function(t, delta, theta)
{
  m=length(theta)
  p.iiplus1=rep(0, (m-1))
  S1.t=FN.S.1(t, theta)
  S1.t.plus.delta=FN.S.1(t=(t+delta), theta)
  p.iiplus1[1]=(FN.S.1(t, theta)-S1.t.plus.delta)/FN.Pi.t(t, theta)[1]
  
  p.iiplus1[2]=(FN.S.2(t, theta, S1.t)-FN.S.2(t=(t+delta), theta, S1.t.plus.delta))/
    FN.Pi.t(t, theta)[2]
  if (is.na(p.iiplus1[2]))
    p.iiplus1[2]=0
  
  return(p.iiplus1)      
}

## Function to calculate the transition matrix according to Scheidegger (2011)
## at age t and delta

FN.P.matrix=function(t, delta, theta) 
{
  m=length(theta)
  P.trans.matrix=matrix(0, nrow=m, ncol=m)
  P.trans.matrix[m,m]=1
  for (j in 1:(m-1))
  {
    P.trans.matrix[j,j+1]=FN.p.iiplus1(t, delta, theta)[j]
    P.trans.matrix[j,j]=1-P.trans.matrix[j,j+1]
  }

  return(P.trans.matrix)    
}  

## Function to calculate a list of transition matrices for all time steps
## from T1+delta (T1=tau=tau2) until tau with a time step size of delta

FN.P.matrix.list=function(T1, delta, theta, tau)
{
  P.matrix.list=list(0)
  t.vec=seq(from=(T1+delta), to=tau, by=delta)
  P.matrix.list[[1]]=FN.P.matrix(t.vec[1], delta, theta)
  for (i in 2:length(t.vec))  
    P.matrix.list[[i]]=FN.P.matrix(t.vec[i], delta, theta)
  return(P.matrix.list)
}

#################################################################################
##
## Function to calculate a vector of probalilties that a pipe is in condition
## state i at (a future) age t=tau given a probability vector at time tau=T1

FN.pi.tau=function(pi.T1, theta, T1, tau, delta)
{
  pi.prev=pi.T1
  
  ## probability vector for time tau
  pi.tau=rep(0, length(theta))
  t.vec=seq(from=(T1+delta), to=tau, by=delta)
  
  for (t in t.vec)
  {
    pi.tau=as.numeric(pi.prev)%*%FN.P.matrix(t, delta, theta)
    pi.prev=pi.tau
  }
  return(pi.tau)
}  

#################################################################################
##
## Function to create matrix of observed condition states at age tau=T1 or tau2=T1
## and forecasted probabilities of condition states at the (future) age tau

FN.matrix.forecast=function(data.set, theta)
{
  forecast=matrix(0, nrow=nrow(data.set), ncol=length(theta)+6)
  col.names=c("Pipe_ID", "T1", "TD", "CS.T1", "tau")
  for (i in 1:length(theta))
    col.names=append(col.names, paste("prob.CS.",i,".tau", sep=""))
  col.names=append(col.names, "CS.tau")
  colnames(forecast)=col.names
  
  forecast=as.data.frame(forecast)
  forecast$Pipe_ID=data.set$Pipe_ID
  forecast$T1=data.set$T1
  forecast$TD=data.set$TD
  forecast$CS.T1=data.set$CS
  
  
  return(forecast)
}


#################################################################################
##
## Functions which randomly assign a CS at the future age tau based on the forecasted 
## probability vector that a pipe will stay in CS i at age tau. Beside the probability 
## vector, a random number sampled from an uniform distribution determines the 
## condition state.

## Assigns an index to an individual pipe depending on the forecasted probability 
## and a random number (used in FN.assign.CS)

FN.assign.CSii=function(y, rand.number)
{
  
  if (rand.number>y)
    y=0
  else y=1
  return(y)
}

FN.assign.CS=function(X)
{
  y=matrix(cumsum(as.numeric(X)))
  CS.vector=as.matrix(rep(NaN,(length(X))))
  rand.number=runif(1, 0, 1)
  CS.vector=apply(y,1,FUN=FN.assign.CSii, rand.number)
  CS.tau=min(which(CS.vector==1))
  CS.tau
  X
  return(CS.tau)
}

#################################################################################
##
## Function which performs a probabilistic forecast of condition states for a whole 
## sewer network described by the matrix "forecast" that a pipe is in CS i at the 
## future age tau and assigns a CS to each pipe based on the probability vector 
## and and a random number

FN.forecast=function(forecast, theta, FN.assign.CS, tau)
{
  
  forecast$tau=tau  
  
  ## Creation of probability matrix for the whole dataset according to the 
  ## observed CS at time T1
  
  prob.matrix.T1=matrix(0, nrow=nrow(forecast), ncol=length(theta))
  
  ## Function which assigns a probability of 1 to the probability matrix
  ## depending on the observed condition state at time T1
  
  FN.assign.prob=function(CS)
  {
    index=CS
    prob.vec=rep(0,length(theta))
    prob.vec[index]=1
    return(prob.vec)
  }
  
  prob.matrix.T1=t(apply(as.matrix(forecast$CS_T1),1,FN.assign.prob))
  
  for (i in 1:nrow(forecast))
  {
    #pi.T1=t(as.matrix(prob.matrix.T1[i,]))
    pi.T1=prob.matrix.T1[i,]
    forecast.input[i,6:(ncol(forecast)-1)]=FN.pi.tau(pi.T1, tau=forecast$tau[i], theta, 
                                                     T1=forecast$T1[i], delta)
  }
  
  ## Assign CS at time tau based on the forecasted probabilities that a pipe stays
  ## in CS i at time tau
  
  forecast$CS.tau=apply(forecast[,6:(ncol(forecast)-1)], 1, FN.assign.CS)
  
  return(forecast)
}

#################################################################################
##
## Function which performs a probabilistic forecast for a whole sewer network
## described by the matrix "forecast" that a pipe is in CS i at time tau and 
## assigns a CS to each pipe based on the probability vector and and a random number

FN.forecast.rehab=function(forecast.input, theta, FN.assign.CS)
{
  
  forecast=forecast.input
  
  ## Creation of probability matrix for the whole dataset according to the 
  ## observed CS at time T1
  
  prob.matrix.t_minus1=matrix(0, nrow=nrow(forecast.input), ncol=length(theta))
  prob.matrix.t=matrix(0, nrow=nrow(forecast.input), ncol=length(theta))
  
  ## Function which assigns a probability of 1 to the probability matrix
  ## depending on the observed condition state at time T1
  
  # Returns a 1x3 vector with the element 1 for the component CS & the 0 for the other components
  FN.assign.prob=function(CS)
  {
    index=CS
    prob.vec=rep(0,length(theta))
    prob.vec[index]=1
    return(prob.vec)
  }
   
  prob.matrix.t_minus1=t(apply(as.matrix(forecast.input$CS_t_minus1),1,FN.assign.prob))
  
  for (i in 1:nrow(forecast.input))
  {
    if (forecast.input$delta_t[i]>0)
    prob.matrix.t[i,]=FN.pi.tau(pi.T1=prob.matrix.t_minus1[i,], theta=theta, 
                                tau=(forecast.input$t_minus1[i]+forecast.input$delta_t[i]), 
                                T1=forecast.input$t_minus1[i], delta) 
  }

  ## Assign CS at the (future) age tau based on the forecasted probabilities that a 
  ## pipe stays in CS i at age tau
  
  for (i in 1:nrow(forecast.input))
  {
    if (forecast.input$delta_t[i]>0)
      forecast$CS_t[i]=FN.assign.CS(prob.matrix.t[i,])
    if (forecast.input$delta_t[i]==0)
      forecast$CS_t[i]= forecast$CS_t_minus1[i]
  }

  forecast$t=forecast$delta_t+forecast$t_minus1
  
  return(forecast)
}


