################################################################################
##
## Script to perform a Bayesian inference of the deterioration 
## (and rehabilitation) parameters of the sewer deterioration model 
## formulated in Egger et al. (2013): 
##
## Egger, C., Scheidegger, A., Reichert, P. and Maurer, M. (2013) 
## Sewer deterioration modeling with condition data lacking historical records. 
## Water Research 47(17), 6762-6779.
##
## The likelihood functions described in Egger et al. (2013) (Eq. 1 und Eq. 6)
## have been extended so that up to two subsequently observed condition states 
## (CS) can be used for inference.
##
## The inference is performed numerically by MCMC. This requires to install
## the R package adaptMCMC. After the inference, the Markov chain is plotted
## as well as prior and posterior marginals
##
## The script consists of the following parts:
##
## 1.) Specification of parameters, input data, etc.
## 2.) Loading functions and data needed for the inference
## 3.) Run inference by MCMC
## 4.) Plot result of inference in terms of Marcov chainsfor analysis
## 
## Please contact Christoph Egger, christoph.egger@hotmail.com for support
## and comments.

## Empty the workspace:
rm(list = ls())

################################################################################
##
## 1.) Please make the following specifications
##
## During the MCMC runs, current samples are subsequently saved. First an
## initial sample is generated. After each saving the sample is extended.

## Set number of samples after which the sample is saved
sampsize=200

## Specify the number extensions to the initial sample. The total sample size
## becomes (n.add+1)*sampsize:
n.add=2

## Optionally an existing sample of parameters can be extended. If so,
## please set "extension" to TRUE:
extension=FALSE

## Specify name of inference
inference="example"

## Specify name of the dataset. 
##
## Each row of the dataset represent one pipe. One row
## has the format:
## tau1 tau2  R D CS1 CS2,
##
## where
##
## tau1 ... Pipe age at the first inspection
## tau2 ... Pipe age at the second inspection
## R ... Pipe age when significant rehabilitation started
## D ... Pipe age when the dataset was lastly updated
## CS1 ... Condition state observed at age tau1
## CS2 ... Condition state observed at age tau2
##
## If only one observed condition state is available, please set
## tau2=CS2=NA for the respective pipes.
##
## The datasets "datset_example_complete" and 
## "dataset_example_incomplete" are provided as example datasets:
##
## - "dataset_example_complete" contains records of all pipes ever built
##   including those which have been rehabilitated (here: replaced) in 
##   the past.
## - "dataset_example_incomplete" contains records of all pipes which
##   currently are in service. Records of pipes which have been rehabilitated
##   (here replaced) in the past have been discarded from the dataset. For
##   this dataset the "conditioned" likelihood is required, see further below.

dataset="dataset_example_incomplete"

## Inference can be done by using the "unconditioned" as well as
## the "conditioned" likelihood according Eq. 1 and Eq. 6 given in
## Egger et al. (2013) "Sewer deterioration modeling with condition 
## data lacking historical records" depending on the availability of
## historical sewer condition data repsective whether rehabilitation
## has taken place or not.

## select type of likelihood ("unconditioned" / "contitioned"):

likelihood.type="conditioned"

## Specify the function used to calculate the log posterior
## included in the script "SDM_functions logposterior.R":

if (likelihood.type=="conditioned")
  name.FN.log.posterior="FN.log.posterior.2CS"

if (likelihood.type=="unconditioned")
  name.FN.log.posterior="FN.log.posterior.2CS.fixed"

## Specify the directory where model functions are located

path.model="C:/SDM/functions"

## Specify the directory where the pipe condition data is located

path.input.data="C:/SDM/input"

## Specify the directory where the prior is located

path.prior="C:/SDM/prior"

## Specify the directory of the result folder

path.result="C:/SDM/result"

## Specify the directory of the R library

path.library="C:/R_library"


  
###########################################################################
##
## 2.) Load functions needed for inference and plotting

setwd(path.model)

source("SDM_functions model.R")
source("SDM_functions logposterior.R")
source("SDM_plotting_functions.R")

## Load the adaptMCMC package
.libPaths(path.library)
library("adaptMCMC")

## Load pipe condition data

setwd(path.input.data)

filename.dataset=paste(dataset, ".RData", sep="")

load(filename.dataset)

## Model cannot handle tau1=0:
data.set$tau1[which(data.set$tau1==0)]=1

## Load prior parameters specifically for the likelihood
## ("conditioned" / "unconditioned")

setwd(path.prior)

load(paste("prior_", likelihood.type, "_likelihood.RData", sep=""))


################################################################################
##
## 3.) Calculation of Markov chain
##
## File name of the result file

file.denotation=paste(dataset, "_", inference, sep="")

result.file.name.Rdata=paste("result_", file.denotation, ".Rdata", sep="")

## Optionally an existing sample of parameters can be extended. Otherwise an
## initial sample is generated.

## Set the selected function to calculate the log posterior
FN.log.posterior=get(name.FN.log.posterior)

## Set start values of the chain
par.start=prior.mean

if (!extension)
{
  if (likelihood.type=="conditioned")
  
  samp=MCMC(p=FN.log.posterior, n=sampsize, init=par.start, scale=0.1*par.start, log=TRUE, 
            adapt=TRUE, acc.rate=0.3, gamma=0.55, list=TRUE, verbose=100, n.start=0, 
            data.set=data.set, prior.mean=prior.mean, prior.sd=prior.sd)
  
  if (likelihood.type=="unconditioned")
   
  samp=MCMC(p=FN.log.posterior, n=sampsize, init=par.start[c(1,2,3)], 
            scale=0.1*par.start[c(1,2,3)], log=TRUE, adapt=TRUE, acc.rate=0.3, 
            gamma=0.55, data.set=data.set, prior.mean=prior.mean, prior.sd=prior.sd)
}

setwd(path.result)

if (extension)
{
  load(result.file.name.Rdata)
}

for (i in 1:n.add)
{
  samp=MCMC.add.samples(samp, n.update=sampsize,data.set=data.set, prior.mean=prior.mean, 
                        prior.sd=prior.sd )
  
  save(file=result.file.name.Rdata, list=c("samp"))
  print(i)
}

################################################################################
##
## 4.) Plot Markov chain as well as prior and posterior marginals
## 
## Plot Markov chain plot
##
## Specify pdf filename of the Markov chain plot

chain.file.name.pdf=paste("chain.SDM.mcmc_", file.denotation, ".pdf", sep="")

names(prior.mean)=c("alpha", "beta", "mu", "lambda[1]",
                    "lambda[2]", "lambda[3]")

pdf(file=chain.file.name.pdf, width=6, height=6)

sysanal.plot.chains(samp$sample)

dev.off()

## Plot prior and posterior marginals
##
## Specify pdf filename of prior and posterior marginals plot

marg.file.name.pdf=paste("marg.SDM.mcmc", file.denotation, ".pdf", sep="")

## Specify the amount of burnin samples. These samples are discarded

amount.burnin.samples=10000

samp.selected=samp$sample[(amount.burnin.samples+1):nrow(samp$sample),]

if (likelihood.type=="conditioned")
{
  mf.row=c(3,2)
  pdf.width=7
}
if (likelihood.type=="unconditioned")
{
  mf.row=c(3,1)
  pdf.width=4
}

mar=c(4,5,3,0)

xlab=list()

for (i in 1:length(prior.mean))
  xlab[[i]]=expression(names(prior.mean))

## Specify "true" parameters if synthetic data is used
par.data.set=NA

FN.plot.marginals(marg.file.name.pdf,
                  prior.mean,
                  prior.sd,
                  par.data.set,
                  samp=samp.selected, 
                  prior.dist="Lognormal",
                  cex.axis=1.5,
                  cex.lab=1.8,
                  xlab,
                  mf.row,
                  pdf.width,
                  pdf.height=8)





