################################################################################
#                                                                              #
# Script extracted from the Systems analysis R library                         #
# ====================================================                         #
#                                                                              #
# Peter Reichert, Eawag, reichert@eawag.ch, last modification Oktober 27, 2015 #
#                                           by Christoph Egger                 #
#                                                                              #
################################################################################

################################################################################
#
# Overview of functions provided by this library
# (see individual descriptions of functions for more details):
#
# Plotting:
# ---------
# sysanal.plot.chains             Plot components of a Markov chain
# sysanal.plot.margs              Plot marginals of a sample
# FN.plot.marginals               Adapted function to plot marginals to specific
#                                 the application case.
#
################################################################################
#
# function to plot markov chains:
# -------------------------------

sysanal.plot.chains <- function(postsamp,ncol=NA,mar=NA,
                                ylim=list(),
                                titles=list(),xlab="chain index",ylab=list())
{
  nvar <- ncol(postsamp)
  if ( is.na(ncol) ) nc <- floor(sqrt(nvar))
  nr <- ceiling(nvar/nc)
  marg <- mar
  if ( is.na(marg[1]) ) marg <- c(2.0,2.0,2.5,0.5) # c(bottom, left, top, right)
  par.def <- par(no.readonly=T)
  par(mfrow=c(nr,nc),xaxs="i",yaxs="i",mar=marg) 
  for ( i in 1:nvar )
  {
    name <- colnames(postsamp)[i]
    data <- postsamp[,i]
    ylim.i <- c(min(data),max(data))
    if ( ylim.i[1] == ylim.i[2] ) 
    {
      ylim.i[1] <- 0.5*ylim.i[1]
      ylim.i[2] <- 1.5*ylim.i[2]
    }
    if ( length(ylim) > 0 )
    {
      ind <- match(name,names(ylim))
      if ( !is.na(ind) )
      {
        ylim.i <- ylim[[ind]]
      }
    }
    title.i <- name
    if ( length(titles) > 0 )
    {
      ind <- match(name,names(titles))
{
        if ( !is.na(ind) )
        {
          title.i <- titles[[ind]]
        }
      }
    }
ylab.i <- name
if ( length(ylab) > 0 )
{
  ind <- match(name,names(ylab))
  if ( !is.na(ind) )
  {
    ylab.i <- ylab[[ind]]
  }
}
plot(data,ylim=ylim.i,type="l",main=title.i,xlab=xlab,ylab=ylab.i)
  }
par(par.def)
}

sysanal.multmatch <- function(x,table,incomparables=NULL)
{
  table.loc <- table
  inds <- numeric(0)
  while (TRUE)
  {
    ind <- match(x,table.loc,incomparables)
    if ( is.na(ind) )
    {
      if ( length(inds) == 0 ) return(ind)
      else                     return(inds)
    }
    inds <- c(inds,ind)
    table.loc[ind] <- paste(x,"_",sep="")
  }  
}

################################################################################

# function to calculate probability densities of univariate distributions:
# ------------------------------------------------------------------------

sysanal.calcpdf <- function(x,distpar,log=FALSE)
{
  if ( distpar[1] == "Uniform" )
  {
    # uniform distribution; parameters are min and max
    min <- as.numeric(distpar[2])
    max <- as.numeric(distpar[3])
    return(dunif(x,min=min,max=max,log=log))
  }
  if ( distpar[1] == "Normal" )
  {
    # normal distribution; parameters are mean and sd:
    mean <- as.numeric(distpar[2])
    sd   <- as.numeric(distpar[3])
    return(dnorm(x,mean=mean,sd=sd,log=log))
  }
  if ( distpar[1] == "NormalTrunc" )
  {
    # truncated normal distribution; parameters are mean, sd, min and max
    mean <- as.numeric(distpar[2])
    sd   <- as.numeric(distpar[3])
    min  <- as.numeric(distpar[4])
    max  <- as.numeric(distpar[5])
    fact <- 1/(pnorm(q=max,mean=mean,sd=sd)-pnorm(q=min,mean=mean,sd=sd))
    if ( !log )
    {
      return(ifelse(x<min|x>max,0,fact*dnorm(x,mean=mean,sd=sd)))
    }
    else
    {
      return(ifelse(x<min|x>max,NA,
                    log(fact)+dnorm(x,mean=mean,sd=sd,log=TRUE)))
    }
  }
  if ( distpar[1] == "Lognormal" )
  {
    # lognormal distribution; parameters are mean and sd mju sigma;
    # R parameters are mean and sd of the log of the random variable m und s
    mean    <- as.numeric(distpar[2])
    sd      <- as.numeric(distpar[3])
    sdlog   <- sqrt(log(1+sd^2/mean^2))
    meanlog <- log(mean) - 0.5*sdlog^2
    return(dlnorm(x,meanlog=meanlog,sdlog=sdlog,log=log))
  }
  if ( distpar[1] == "LognormalTrunc" )
  {
    # truncated lognormal distribution; parameters are mean, sd, min and max;
    # R parameters are mean and sd of the log of the random variable
    mean    <- as.numeric(distpar[2])
    sd      <- as.numeric(distpar[3])
    sdlog   <- sqrt(log(1+sd^2/mean^2))
    meanlog <- log(mean) - 0.5*sdlog^2
    min     <- as.numeric(distpar[4])
    max     <- as.numeric(distpar[5])
    fact <- 1/(plnorm(q=max,meanlog=meanlog,sdlog=sdlog)-plnorm(q=min,meanlog=meanlog,sdlog=sdlog))
    if ( !log )
    {
      return(ifelse(x<min|x>max,0,fact*dlnorm(x,meanlog=meanlog,sdlog=sdlog)))
    }
    else
    {
      return(ifelse(x<min|x>max,NA,
                    log(fact)+dlnorm(x,meanlog=meanlog,sdlog=sdlog,log=TRUE)))
    }
  }
  if ( distpar[1] == "Inv" )
  {
    # inverse distribution; parameters are min and max:
    min <- as.numeric(distpar[2])
    max <- as.numeric(distpar[3])
    if ( !log )
    {
      return(ifelse(x<min|x>max,0,1/(log(max/min)*x)))   
    }
    else
    {
      return(ifelse(x<min|x>max,NA,-log(log(max/min)) - log(x)))   
    }
  }
  if ( distpar[1] == "Exponential" )
  {
    # exponential distribution; parameter is mean:
    mean <- as.numeric(distpar[2])
    if ( !log )
    {
      return(ifelse(x<0,0,1/mean*exp(-x/mean)))   
    }
    else
    {
      return(ifelse(x<0,NA,-log(mean)-x/mean))   
    }
  }
  stop(paste("Distribution",dist,"not yet implemented"))
}


################################################################################

# function to plot prior and posterior marginals:
# -----------------------------------------------

sysanal.plot.margs <- function(postsamp,pridist=list(),names.var, ncol=NA,mar=NA,
                               xlim=list(),ymax=list(), 
                               oma=oma,mfrow=mf.row,
                               titles=list(),xlab=list(),ylab=list(),
                               lty=NA, cex.lab=cex.lab, cex.axis=cex.axis,
                               ...)
{
  postsamp.list <- list()
  if ( is.data.frame(postsamp) )
  {
    postsamp.list[[1]] <- postsamp
  }else
  {
    if ( is.matrix(postsamp) )
    {
      postsamp.list[[1]] <- postsamp
    }
    else
    {
      if ( is.list(postsamp) )  # be careful: a data frame is a list!
      {
        postsamp.list <- postsamp
      }
      else
      {
        stop("sysanal.plot.margs: postsamp is of illegal type")
      }
    }
  }
  
  nsamp <- length(postsamp.list)
  var <- colnames(postsamp.list[[1]])
  if ( nsamp > 1 )
  {
    for ( j in 2:nsamp ) 
    {
      var <- c(var,colnames(postsamp.list[[j]]))
    }
    var <- unique(var)
  }
  nvar <- length(var)
  
  if ( !is.na(ncol) ) nc <- ncol
  else                nc <- floor(sqrt(nvar))
  nr <- ceiling(nvar/nc)
  marg <- mar
  if ( is.na(marg[1]) ) marg <- c(3.0,3.0,3.5,0.5) # c(bottom, left, top, right)
  
  lty.loc <- lty
  if ( length(lty) <  nsamp+1 ) lty.loc <- 1:(nsamp+1)
  
  par.def <- par(no.readonly=T)
  par(mfrow=mf.row,xaxs="i",yaxs="i",mar=marg, oma=oma) # c(bottom, left, top, right)
  for ( i in 1:nvar )
  {
    name <- names.var[i]
    data.min    <- NA
    data.max    <- NA
    density.max <- NA
    d <- as.list(rep(NA,nsamp))
    for ( j in 1:nsamp )
    {
      ind <- match(var[i],colnames(postsamp.list[[j]]))
      if ( !is.na(ind) )
      {
        data.min    <- min(c(data.min,postsamp.list[[j]][,ind]),na.rm=TRUE)
        data.max    <- max(c(data.max,postsamp.list[[j]][,ind]),na.rm=TRUE)
        d[[j]]      <- density(postsamp.list[[j]][,ind],adjust=1.0)
        density.max <- max(c(density.max,d[[j]]$y),na.rm=TRUE)
      }
    }
    
    xlim.i <- c(data.min,data.max)
    #xlim.i <- c(par.data.set[i]-0.001,data.max)
    if ( xlim.i[1] == xlim.i[2] ) 
    {
      xlim.i[1] <- 0.5*xlim.i[1]
      xlim.i[2] <- 1.5*xlim.i[2]
    }
    if ( length(xlim) > 0 )
    {
      ind <- match(name,names(xlim))
      if ( !is.na(ind) )
      {
        xlim.i <- xlim[[ind]]
      }
    }
    
    if ( is.na(density.max) ) ylim.i <- c(0,1) else   ylim.i <- c(0,1.1*density.max)
    if ( length(ymax) > 0 )
    {
      ind <- match(name,names(ymax))
      if ( !is.na(ind) )
      {
        ylim.i <- c(0,ymax[[ind]])
      }
    }
    
    title.i <- name
    if ( length(titles) > 0 )
    {
      ind <- match(name,names(titles))
{
        if ( !is.na(ind) )
        {
          title.i <- titles[[ind]]
        }
      }
    }

xlab.i <- parse(text=name)
if ( length(xlab) > 0 )
{
  ind <- match(name,names(xlab))
  if ( !is.na(ind) )
  {
    xlab.i <- parse(xlab[[ind]])
  }
}

ylab.i <- "Probability density (-)"
if ( length(ylab) > 0 )
{
  ind <- match(name,names(ylab))
  if ( !is.na(ind) )
  {
    ylab.i <- ylab[[ind]]
  }
}

plot(numeric(0),numeric(0),xlab=xlab.i,ylab=ylab.i,
     xlim=xlim.i,ylim=ylim.i,type="n", cex.axis=cex.axis, cex.lab=cex.lab)


if (!is.na(par.data.set[i]))
  lines(x=c(par.data.set[i], par.data.set[i]), y=c(0,  1000), col="blue", lwd=2)

## plots posterior pdf
for ( j in 1:nsamp )
{
  if ( !is.na(d[[j]])[[1]] )
  {
    lines(d[[j]],lty=lty.loc[j])
    #lines(x=rep(mean(postsamp.list[[1]][,i]),2), y=c(0, (max(d[[j]]$y)+1000)), lty=lty.loc[j], lwd=2)
  }
}

## plots prior pdf
x <- seq(xlim.i[1],xlim.i[2],length=101)
y <- numeric(0)
if ( length(pridist) > 0 )
{
  ind <- sysanal.multmatch(name,names(pridist))
  if ( !is.na(ind[1]) )
  {
    for ( j in 1:length(ind) )
    {
      y <- sysanal.calcpdf(x,pridist[[ind[j]]])
      lines(x,y,lty=lty.loc[nsamp+1])
    }
  }
}
  }
par(par.def)
}


################################################################################
##
## Function to plot prior and posterior marginals adapted by Christoph Egger

FN.plot.marginals=function(marg.file.name.pdf, prior.mean, prior.sd, 
                           samp, prior.dist, par.data.set,cex.axis, cex.lab,
                           xlab, mf.row, pdf.width, pdf.height)
  
{
  prior.margs.def <- list()
  xlim            <- list()
  
  for ( j in 1:length(prior.mean))
    {
       prior.margs.def[[j]] <- c(prior.dist, prior.mean[j],prior.sd[j])
       xlim[[j]]            <- prior.mean[j]+c(-10,8)*prior.sd[j]
    }
   
  names(prior.margs.def) <-names(prior.mean)
    
  ## Plot prior and posterior marginals
   
  pdf(file=marg.file.name.pdf, width=pdf.width, height=pdf.height)
  sysanal.plot.margs(postsamp=samp,names.var=names(prior.margs.def),
                     pridist=prior.margs.def,
                     xlim=xlim,
                     xlab=xlab,
                     oma=c(1,1,1,1)+0.1,
                      mar=mar,
                      par.data.set=par.data.set,
                     cex.lab=cex.lab, cex.axis=cex.axis,
                     mf.row=mf.row
                     #posterior.sd=posterior.sd
                     )
  dev.off()
}


